"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_errors_1 = require("apollo-server-errors");
const apollo_server_env_1 = require("apollo-server-env");
const predicates_1 = require("../utilities/predicates");
class RemoteGraphQLDataSource {
    constructor(config) {
        if (config) {
            return Object.assign(this, config);
        }
    }
    process({ request, context, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const headers = new apollo_server_env_1.Headers();
            headers.set('Content-Type', 'application/json');
            request.http = {
                method: 'POST',
                url: this.url,
                headers,
            };
            if (this.willSendRequest) {
                yield this.willSendRequest({ request, context });
            }
            const { http } = request, graphqlRequest = __rest(request, ["http"]);
            const options = Object.assign({}, http, { body: JSON.stringify(graphqlRequest) });
            const httpRequest = new apollo_server_env_1.Request(request.http.url, options);
            try {
                const httpResponse = yield apollo_server_env_1.fetch(httpRequest);
                const body = yield this.didReceiveResponse(httpResponse, httpRequest);
                if (!predicates_1.isObject(body)) {
                    throw new Error(`Expected JSON response body, but received: ${body}`);
                }
                const response = Object.assign({}, body, { http: httpResponse });
                return response;
            }
            catch (error) {
                this.didEncounterError(error, httpRequest);
                throw error;
            }
        });
    }
    didReceiveResponse(response, _request) {
        return __awaiter(this, void 0, void 0, function* () {
            if (response.ok) {
                return this.parseBody(response);
            }
            else {
                throw yield this.errorFromResponse(response);
            }
        });
    }
    didEncounterError(error, _request) {
        throw error;
    }
    parseBody(response) {
        const contentType = response.headers.get('Content-Type');
        if (contentType && contentType.startsWith('application/json')) {
            return response.json();
        }
        else {
            return response.text();
        }
    }
    errorFromResponse(response) {
        return __awaiter(this, void 0, void 0, function* () {
            const message = `${response.status}: ${response.statusText}`;
            let error;
            if (response.status === 401) {
                error = new apollo_server_errors_1.AuthenticationError(message);
            }
            else if (response.status === 403) {
                error = new apollo_server_errors_1.ForbiddenError(message);
            }
            else {
                error = new apollo_server_errors_1.ApolloError(message);
            }
            const body = yield this.parseBody(response);
            Object.assign(error.extensions, {
                response: {
                    url: response.url,
                    status: response.status,
                    statusText: response.statusText,
                    body,
                },
            });
            return error;
        });
    }
}
exports.RemoteGraphQLDataSource = RemoteGraphQLDataSource;
//# sourceMappingURL=RemoteGraphQLDatasource.js.map